app.DataSource = Class.extend({
	/**
	 * main url path of the application
	 */
	url: '/FromScratch/cxfservlet/s2/chat/',

	coords: {
		lat: 0,
		long: 0
	},

	/**
	 * @constructor
	 */
	init: function () {
		this.getData();

		//get position at first time
		app.getPosition($.proxy(this.onCoordsUpdate, this));

		//start an interval to get position data
		setInterval($.proxy(function () {
			app.getPosition($.proxy(this.onCoordsUpdate, this));
		}, this), 60000);
	},

	/**
	 * Set coords value on API response
	 * @param position
	 */
	onCoordsUpdate: function (position) {
		this.coords.lat = position.coords.latitude;
		this.coords.long = position.coords.longitude;
	},

	/**
	 * Start polling
	 */
	poll: function () {
		this.timeout = setTimeout($.proxy(function () {
			this.getData();
		}, this), 1000);
	},

	/**
	 * Restart polling
	 */
	clear: function () {
		this.ajax && this.ajax.abort();
		clearTimeout(this.timeout);

		this.poll();
	},

	/**
	 * @method getData
	 */
	getData: function () {
		this.ajax = $.ajax({
			url: this.url + 'poll',
			type: "POST",
			data: {
				lat: this.coords.lat,
				lon: this.coords.long
			},
			success: $.proxy(this.onDataResponse, this),
			error: $.proxy(this.onError, this)
		});
	},

	/**
	 * @method onDataResponse
	 * @param resp
	 */
	onDataResponse: function (resp) {
		app.model.setData(resp);

		this.poll();
	},

	onError: function () {
		this.poll();
	},

	/**
	 * Create a conversation on the server, and push it to the model data
	 * @param arr
	 */
	createConversation: function (arr) {
		arr.push(app.model.get('currentUser').id);

		$.post(this.url + 'start', {
			users: arr
		});

		this.clear();
	},

	/**
	 * open a conversation on the server
	 * @param conversation
	 */
	openConversation: function (conversation) {
		$.post(this.url + 'addusers', {
			conversation: conversation.id,
			users: [app.model.get('currentUser').id]
		});

		this.clear();
	},

	/**
	 * leave conversation
	 */
	leaveConversation: function () {
		$.post(this.url + 'leave', {
			conversation: app.model.get('conversation')
		});

		this.clear();
	},

	/**
	 * send a message
	 * @param msg
	 */
	sendMessage: function (msg) {
		$.post(this.url + 'send', {
			conversation: app.model.get('conversation'),
			message: msg
		});

		this.clear();
	},

	/**
	 * set a conversation to public
	 * @param public
	 */
	setPublic: function (public) {
		$.post(this.url + 'setpublic', {
			conversation: app.model.get('conversation'),
			public: public
		});

		this.clear();
	}
});