app.Model = Class.extend({
	/**
	 * @property observers
	 * @type {Object}
	 */
	observers: {},

	/**
	 * @property data
	 * @type {Object}
	 */
	data: {
		users: {},
		openedConversations: [],
		publicConversations: [],
		messages: {}
	},

	/**
	 * @constructor
	 */
	init: function () {},

	/**
	 * @method set
	 * @param key
	 * @param data
	 */
	set: function (key, data) {
		this.data[key] = data;
		this.changed(key);
	},

	/**
	 * @method get
	 * @param key
	 * @returns {*}
	 */
	get: function (key) {
		return this.data[key];
	},

	/**
	 * @method setData
	 * @param data
	 */
	setData: function (data) {
		this.set('currentUser', data.currentUser);

		this.setUsers(data.users, data.activeUsers);
		this.setConversations(data.openedConversations, data.conversations);
		this.setMessages(data.messages);
	},

	/**
	 * @method setUsers
	 * @param users
	 * @param activeUsers
	 */
	setUsers: function (users, activeUsers) {
		var obj = {};

		$.each(users, function (i, e) {
			if ($.inArray(e.id, activeUsers) !== -1) {
				e.status = 'online';
			} else {
				e.status = '';
			}

			obj[e.id] = e;
		});

		this.set('users', obj);
	},

	/**
	 * @method setConversations
	 * @param opened
	 * @param all
	 */
	setConversations: function (opened, all) {
		var openedArr = [],
			publicArr = [];

		$.each(all, $.proxy(function (i, e) {
			e.title = this.getConversationTitle(e);

			if ($.inArray(e.id, opened) !== -1) {
				openedArr.push(e);
			} else {
				publicArr.push(e);
			}
		}, this));

		this.set('openedConversations', openedArr);
		this.set('publicConversations', publicArr);
	},

	/**
	 * @method openConversation
	 * @param id
	 */
	openConversation: function (id) {
		var arr = [],
			elem,
			opened = this.get('openedConversations');

		$.each(this.get('publicConversations'), function (i, e) {
			if (e.id == id) {
				elem = e;
				return;
			}

			arr.push(e);
		});

		opened.push(elem);

		app.dataSource.openConversation(elem);

		this.set('publicConversations', arr);
		this.set('openedConversations', opened);
	},

	/**
	 * @method leaveCurrentConversation
	 */
	leaveCurrentConversation: function () {
		var arr = [],
			elem,
			closed = this.get('publicConversations');

		app.dataSource.leaveConversation();

		$.each(this.get('openedConversations'), $.proxy(function (i, e) {
			if (e.id == this.get('conversation')) {
				elem = e;
				return;
			}

			arr.push(e);
		}, this));

		closed.push(elem);

		this.set('openedConversations', arr);
		this.set('publicConversations', closed);

		this.set('conversation', false);
	},

	getConversationTitle: function (conversation) {
		var names = [];
		$.each(conversation.userIds, function (ind, user) {
			var usr = app.model.get('users')[user];

			if (usr.id === app.model.get('currentUser').id) {
				names.push('<b>me</b>');
			} else {
				names.push(usr.nickname);
			}
		});

		return names.join(' & ');
	},

	/**
	 * @method setMessages
	 * @param messages
	 */
	setMessages: function (messages) {
		var obj = this.get('messages') || {};
		$.each(messages, function (i, e) {
			var counter = $('#conversation-list li[data-id="' + e.conversationId + '"]').find('.counter');

			if (!obj[e.conversationId]) {
				obj[e.conversationId] = [];
			}

			if (parseInt(e.conversationId) !== parseInt(app.model.get('conversation'))) {
				counter.html(parseInt(counter.html() || 0) + 1);
			} else {
				counter.html('');
			}

			obj[e.conversationId].push(e);
		});

		this.set('messages', obj);
	},
	
	/**
	 * @method changed
	 * @param key
	 */
	changed: function (key) {
		if (!this.observers[key]) {
			return;
		}

		$.each(this.observers[key], $.proxy(function (i, e) {
			e.func.call(e.ctx, this.data[key]);
		}, this));
	},

	/**
	 * @method on
	 * @param key
	 * @param func
	 * @param ctx
	 */
	on: function (key, func, ctx) {
		if (!this.observers[key]) {
			this.observers[key] = [];
		}

		this.observers[key].push({
			func: func,
			ctx: ctx || this
		})
	}
});