app.Conversation = app.View.extend({
	container: $('#conversation'),

	emoticonMap: {
		":'(": 'Cry',
		":(": 'Frown',
		"(:|": 'Not-Amused',
		":|": 'Ambivalent',
		":@": 'Angry',
		":S": 'Confused',
		":8": 'Cool',
		":P": 'Crazy',
		":O": 'Gasp',
		"(L)": 'Heart',
		":D": 'Laughing',
		";)": 'Wink',
		":/": 'Slant',
		":)": 'Smile',
		"(n)": 'Thumbs-Down',
		"(y)": 'Thumbs-Up'
	},

	/**
	 * @property template
	 */
	template: '<li data-id="{id}">' +
		'<span class="username">' +
		'<img src="{image}" width="30" height="30" />' +
		'{nickname}' +
		'</span>' +
		'<span class="msg">{content}</span>' +
	'</li>',

	/**
	 * @constructor
	 */
	init: function () {
		//observe model properties
		app.model.on('conversation', $.proxy(this.onConversationChange, this));
		app.model.on('messages', $.proxy(this.onNewMessage, this));

		//save containers
		this.input = $('#new-msg');
		this.smileyContainer = this.container.find('.smiley-container');

		//bind our events
		this.container
			.on('keydown', '#new-msg', $.proxy(this.onPost, this))
			.on('click', '#leaveConversation', $.proxy(this.onLeave, this))
			.on('click', '.public .checkbox', $.proxy(this.onPublicSet, this))
			.on('click', '.smiley-container .button', $.proxy(this.onSmileyContainerClick, this))
			.on('click', '.smiley-container img', $.proxy(this.onSmileyClick, this));
	},

	/**
	 * public button set handler
	 * @param e
	 */
	onPublicSet: function (e) {
		var checkbox = $(e.target).closest('.checkbox'),
			selected;

		checkbox.toggleClass('selected');

		selected = checkbox.hasClass('selected');

		//send it to the datasource
		app.dataSource.setPublic(selected);
	},

	/**
	 * @method onNewMessage
	 * @param messages
	 */
	onNewMessage: function (messages) {
		if (!this.conversation) return;

		messages = messages[this.conversation];

		if (!messages) {
			return;
		}

		var html = '';

		//Assembly our message list
		$.each(messages, $.proxy(function (i, e) {
			if (this.container.find('li[data-id="' + e.id + '"]').length) {
				return;
			}

			var usr = app.model.get('users')[e.userId],
				nickname = usr.nickname;

			html += this.getTemplate(e, {
				nickname: app.model.get('currentUser').id === usr.id ? '<b>me</b>' : nickname,
				image: '/FromScratch/cxfservlet/s2/reg/images/' + nickname + '.jpg'
			});
		}, this));

		//append it to the dom
		this.container.find('ul').append(html);

		//Animate to the top of the window
		$(document.body).animate({
			scrollTop: $(document.body).height()
		})
	},

	/**
	 * set Public action
	 * @param pub
	 */
	setPublic: function (pub) {
		var checkbox = this.container.find('.checkbox');

		if (pub) {
			checkbox.addClass('selected');
		} else {
			checkbox.removeClass('selected');
		}
	},

	/**
	 * @method onConversationChange
	 */
	onConversationChange: function (data) {
		if (!data) {
			this.container.addClass('empty');
			this.container.find('ul').html('');
			return;
		}

		this.conversation = data;

		this.container
			.removeClass('empty')
			.find('ul').html('')
			.end()
			.find('.title .text').html('');
	},

	/**
	 * @method onPost
	 */
	onPost: function (e) {
		if (e.keyCode !== 13 || !app.model.get('conversation')) return;

		app.dataSource.sendMessage($(e.target).val());

		$(e.target).val('');
	},

	/**
	 * leave conversation
	 */
	onLeave: function () {
		app.model.leaveCurrentConversation();
	},

	/**
	 * Decorate the given data with emoticons
	 * @param data
	 * @returns {*}
	 */
	decorateData: function (data) {
		$.each(this.emoticonMap, function (i, e) {
			data.content = data.content.replace(i, '<img src="assets/img/emoticons/' + e + '.png"/>');
		});

		return data;
	},

	/**
	 * we have to open smiley list on container click
	 */
	onSmileyContainerClick: function () {
		this.smileyContainer.toggleClass('opened');
		this.input.focus();
	},

	/**
	 * we add a smiley to the textarea when smiley icon is clicked
	 * @param e
	 */
	onSmileyClick: function (e) {
		var caretPos = this.input[0].selectionStart,
			textAreaTxt = this.input.val(),
			txtToAdd = $(e.target).data('text');

		this.input.val(textAreaTxt.substring(0, caretPos) + txtToAdd + textAreaTxt.substring(caretPos)).focus();

		this.onSmileyContainerClick();
	},

	addMarker: function () {

	}
});