app.ConversationList = app.View.extend({
	/**
	 * @property template
	 */
	template: '<li class="{clss}" data-id="{id}" data-public="{public_}">' +
		'<p>{title}</p><span class="counter"></span>' +
		'<span class="bttn-small">{label}</span>' +
		'</li>',

	/**
	 * @constructor
	 */
	init: function (container, opened) {
		this.container = $(container);
		this.opened = opened;
		this.label = this.opened ? 'Show' : 'Join';

		//Conversation click handler
		this.container.on('click', 'li .bttn-small', $.proxy(this.onConversationClick, this));

		//We observe conversations from the model
		app.model.on(
			this.opened ? 'openedConversations' : 'publicConversations',
			$.proxy(this.onListChange, this)
		);

		//We also observe the current conversation
		app.model.on('conversation', $.proxy(this.onConversationChange, this));
	},

	/**
	 * @method onConversationClick
	 * @param e
	 */
	onConversationClick: function (e) {
		var target = $(e.target).closest('li'),
			id = target.data('id');

		if (!this.opened) {
			app.model.openConversation(id);
		}

		target.find('.counter').html('');

		app.model.set('conversation', target.data('id'));

		app.views.conversation.setPublic(!!target.data('public'));
	},

	/**
	 * If conversation changes in the model, we update this view
	 * @param conversation
	 */
	onConversationChange: function (conversation) {
		this.container
			.find('.active').removeClass('active')
			.end()
			.find('li[data-id="' + conversation + '"]')
			.addClass('active');
	},

	/**
	 * @method onRoomsChange
	 * @param conversations
	 */
	onListChange: function (conversations) {
		var newItems = [],
			convIds = [];

		//We do not change the whole list, we dynamically search for changes
		$.each(conversations, $.proxy(function (i, e) {
			var elem = this.container.find('li[data-id="' + e.id + '"]');

			if (elem.length) {
				elem.find('p').html(e.title);
				elem.attr('data-public', e.public_);
			} else {
				newItems.push(e);
			}

			convIds.push(e.id);
		}, this));

		//Insert new items
		this.insertNewItems(newItems);
		//Delete removed ones
		this.removeDeleted(convIds);
	},

	/**
	 * Insert new conversations to the list
	 * @param newItems
	 */
	insertNewItems: function (newItems) {
		if (!newItems || !newItems.length) return;

		var html = '';

		$.each(newItems, $.proxy(function (i, e) {
			html += this.getTemplate(e, {
				clss: this.clss,
				label: this.label
			});
		}, this));

		this.container.append(html);
	}
});