app.UserList = app.View.extend({
	container: $('#user-list'),

	/**
	 * @property template
	 */
	template: '<li data-id="{id}" data-status="{status}">' +
		'<span class="checkbox"><i></i></span>' +
		'<span class="name">{nickname}</span>' +
		'<span class="status">online</span>' +
		'</li>',

	/**
	 * @constructor
	 */
	init: function () {
		this.createButton = this.container.find('#createConversation');
		this.map = $('#map');
		this.profile = $('#profile');

		this.container.on('click', '.checkbox', $.proxy(this.onUserSelect, this));
		this.container.on('click', 'li', $.proxy(this.onUserClick, this));
		this.createButton.on('click', $.proxy(this.onCreate, this));

		 //We observe model users data
		app.model.on('users', $.proxy(this.onUsersChange, this));

		this.profile.on('click', '.close', function () {
			$(this).parent().removeClass('opened');
		});
	},

	/**
	 * Conversation create
	 */
	onCreate: function () {
		var selected = this.container.find('.selected'),
			arr = [];

		selected.each(function (i, e) {
			arr.push($(e).data('id'));
		});

		//we set it via datasource
		app.dataSource.createConversation(arr);

		selected.removeClass('selected');

		//set create button state
		this.setCreateButton();
	},

	/**
	 * User checkbox input change handler, we return on owner click
	 * @param e
	 */
	onUserSelect: function (e) {
		var li = $(e.target).closest('li');

		if (li.hasClass('owner')) return;

		li.toggleClass('selected');

		this.setCreateButton();
	},

	/**
	 * we open user page on user click
	 * @param e
	 */
	onUserClick: function (e) {
		var target = $(e.target),
			id, userData, latlng;

		if (target.closest('.checkbox').length) {
			return;
		}

		target = target.closest('li');
		id = target.data('id');
		userData = 	app.model.data.users[id];
		latlng = new google.maps.LatLng(userData.lat, userData.lon);

		 //set data properties from the model by data-* attributes in the DOM
		$.each(userData, $.proxy(function (i, e) {
			this.profile.find('[data-' + i + ']').html(e);
		}, this));

		//Remove markers
		if (this.marker) {
			this.marker.setMap(null);
		}

		//Create the current user's location marker
		this.marker = new google.maps.Marker({
			position: latlng,
			map: app.map,
			title: userData.nickname
		});

		//set it on the map
		app.map.setCenter(latlng);

		//show the profile container
		this.profile.addClass('opened');
	},

	/**
	 * create button state handler
	 */
	setCreateButton: function () {
		if (this.container.find('.selected').length > 0) {
			this.createButton.addClass('enabled');
		} else {
			this.createButton.removeClass('enabled');
		}
	},

	/**
	 * @method onUsersChange
	 * @param users
	 */
	onUsersChange: function (users) {
		var newItems = [],
			userIds = [];

		//we set user values
		$.each(users, $.proxy(function (i, e) {
			var elem = this.container.find('li[data-id="' + e.id + '"]');

			elem.attr('data-status', e.status);

			if (elem.length) {
				elem.find('.name').text(e.nickname);
			} else {
				newItems.push(e);
			}

			userIds.push(e.id);
		}, this));

		//insert / remove elements
		this.insertNewItems(newItems);
		this.removeDeleted(userIds);

		this.container
			.find('li[data-id="' + app.model.get('currentUser').id + '"]')
			.addClass('owner');
	},

	/**
	 * insert new items
	 * @param newItems
	 */
	insertNewItems: function (newItems) {
		if (!newItems || !newItems.length) return;

		var html = '';

		$.each(newItems, $.proxy(function (i, e) {
			html += this.getTemplate(e);
		}, this));

		this.container.find('ul').append(html);
	}
});